/// <reference types="node" />
import { IRouter, NextFunction, Request, Response } from "express";
import { ApplicationShtudownStatus, ApplicationStartupStatus, ApplicationStatus } from "./enum-application-life-cycle-status";
import { DisposableSingletonContainer } from "./disposable-singleton-container";
import { IProbe } from "./i-probe";
import { IProbeResult } from "./i-probe-result";
declare module 'express' {
    interface Express {
        [Symbol.asyncDispose]?: () => Promise<void>;
    }
}
/**
 * The ApplicationBuilder class is responsible for building an express application.
 * It sets up the application's health status, ports, middlewares, routers, and error handling.
 *
 * @property {K8SHealthStatus} healthStatus - The health status of the application.
 * @property {string} applicationName - The name of the application.
 * @property {any} swaggerDocument - The swagger document for the application(JSON representation).
 */
export declare class ApplicationBuilder {
    applicationName: string;
    swaggerDocument: any;
    startupHandler: (rootRouter: IRouter, DIContainer: DisposableSingletonContainer) => Promise<IProbeResult<ApplicationStartupStatus>>;
    shutdownHandler: () => Promise<IProbeResult<ApplicationShtudownStatus>>;
    livenessProbe: IProbe<ApplicationStatus>;
    readinessProbe: IProbe<ApplicationStatus>;
    private readonly currentProcess;
    private readonly exitSignals;
    private readonly container;
    private applicationStatus;
    private applicationPort;
    private healthPort;
    private appMiddlewares;
    private appRouters;
    private helmetMiddleware;
    private bodyParserUrlEncodingMiddleware;
    private bodyParserJsonMiddleware;
    private catchAllErrorResponseTransformer;
    private readonly exitHandler;
    /**
     * Creates an instance of ApplicationBuilder.
     *
     * @param {string} applicationName - The name of the application.
     * @param {any} swaggerDocument - The swagger document for the application(JSON representation).
     * @param startupHandler - The startup handler that has to be invoked before application starts, used to indicate the application's startup status.
     * @param shutdownHandler - The shutdown handler that has to be invoked before application shutdowns, used to indicate the application's livelines status.
     * @param {IProbe} livenessProbe - The liveness probe used to indicate the application's liveness status.
     * @param {IProbe} readinessProbe - The readiness probe used to indicate the application's readiness status.
     * @param {NodeJS.Process} currentProcess - The current process.
     * @param {NodeJS.Signals[]} exitSignals - The exit signals.
     * @param {DisposableSingletonContainer} container - The container for disposable singletons.
     */
    constructor(applicationName?: string, swaggerDocument?: any, startupHandler?: (rootRouter: IRouter, DIContainer: DisposableSingletonContainer) => Promise<IProbeResult<ApplicationStartupStatus>>, shutdownHandler?: () => Promise<IProbeResult<ApplicationShtudownStatus>>, livenessProbe?: IProbe<ApplicationStatus>, readinessProbe?: IProbe<ApplicationStatus>, currentProcess?: NodeJS.Process, exitSignals?: NodeJS.Signals[], container?: DisposableSingletonContainer);
    /**
     *  Used to override the startup handler.
     * @param startupHandler Handler to be invoked before application starts, used to indicate the application's startup status.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideStartupHandler(startupHandler: () => Promise<IProbeResult<ApplicationStartupStatus>>): ApplicationBuilder;
    /**
     * Used to override the shutdown handler.
     * @param shutdownHandler Handler to be invoked before application shutdowns, used to cleanup reasources.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideShutdownHandler(shutdownHandler: () => Promise<IProbeResult<ApplicationShtudownStatus>>): ApplicationBuilder;
    /**
     * Used to override the liveness probe.
     * @param livenessProbe Probe used to indicate the application's liveness status.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideLivenessProbe(livenessProbe: IProbe<ApplicationStatus>): ApplicationBuilder;
    /**
     * Used to override the readiness probe.
     * @param readinessProbe Probe used to indicate the application's readiness status.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideReadinessProbe(readinessProbe: IProbe<ApplicationStatus>): ApplicationBuilder;
    /**
     *  Used to overrides the application port default(3000).
     * @param {number} port new application port.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideAppPort(port: number): ApplicationBuilder;
    /**
     *  Used to overrides the health port default(5678).
     * @param {number} port new health port.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideHealthPort(port: number): ApplicationBuilder;
    /**
     * Used to overrides the helmet configuration.
     * @param {(request: Request, response: Response, next: NextFunction) => void} helmet new helmet configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideHelmetConfiguration(helmet: (request: Request, response: Response, next: NextFunction) => void): ApplicationBuilder;
    /**
     * Used to overrides the bodyParserUrlEncoding configuration.
     * @param {(request: Request, response: Response, next: NextFunction) => void} bodyParserUrlEncoding new bodyParserUrlEncoding configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideBodyParserUrlEncodingConfiguration(bodyParserUrlEncoding: (request: Request, response: Response, next: NextFunction) => void): ApplicationBuilder;
    /**
     * Used to overrides the bodyParserJson configuration.
     * @param {(request: Request, response: Response, next: NextFunction) => void} bodyParserJson new bodyParserJson configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideBodyParserJsonConfiguration(bodyParserJson: (request: Request, response: Response, next: NextFunction) => void): ApplicationBuilder;
    /**
     * Used to overrides the catchAllErrorResponseTransformer configuration.
     * @param {(request: Request, error: unknown) => unknown} transformer new catchAllErrorResponseTransformer configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideCatchAllErrorResponseTransformer(transformer: (request: Request, error: unknown) => unknown): ApplicationBuilder;
    /**
    * Used to register a SYNC/ASYNC middleware.
    * @param {(request: Request, response: Response, next: NextFunction) => Promise<void>|void} middleware middleware to be registered.
    * @returns {ApplicationBuilder} ApplicationBuilder instance.
    */
    registerApplicationMiddleware(middleware: (request: Request, response: Response, next: NextFunction) => Promise<void> | void): ApplicationBuilder;
    /**
     * Used to register a router.
     * @param {string} path path for the router.
     * @param {IRouter} router router to be registered.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    registerApplicationRoutes(path: string, router: IRouter): ApplicationBuilder;
    /**
     * Used to start the application using the configured parameters.
     * @returns {Promise<void>} Promise that resolves when the application is started.
     */
    start(): Promise<void>;
    /**
     * Used to stop the application. This clears all the middlewares and routers.(all config is reset to default)
     * @returns {Promise<void>} Promise that resolves when the application is stopped.
     */
    [Symbol.asyncDispose](): Promise<void>;
    private appExpressListen;
    private healthExpressListen;
    private checkHealthStatus;
    private errorHandler;
}
